@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -65,6 +65,7 @@
 #include "MagickCore/statistic.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
+#include "coders/coders-private.h"
 
 /*
   Typedef declarations.
@@ -259,12 +260,17 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 { \
   if (comment_info.comment != (char *) NULL)  \
     comment_info.comment=DestroyString(comment_info.comment); \
+  if (quantum_info != (QuantumInfo *) NULL) \
+    quantum_info=DestroyQuantumInfo(quantum_info); \
   ThrowReaderException((exception),(message)); \
 }
 
   char
     format;
 
+  ColorspaceType
+    colorspace;
+
   CommentInfo
     comment_info;
 
@@ -278,6 +284,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image;
 
   MagickBooleanType
+    is_gray = MagickTrue,
+    is_mono = MagickTrue,
     status;
 
   QuantumAny
@@ -291,8 +299,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   size_t
     depth,
-    extent,
-    packet_size;
+    extent;
 
   ssize_t
     count,
@@ -307,11 +314,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -322,6 +329,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read PNM image.
   */
+  comment_info.comment=(char *) NULL;
+  quantum_info=(QuantumInfo *) NULL;
   count=ReadBlob(image,1,(unsigned char *) &format);
   do
   {
@@ -333,6 +342,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((count != 1) || (format != 'P'))
       ThrowPNMException(CorruptImageError,"ImproperImageHeader");
     max_value=1;
+    colorspace=UndefinedColorspace;
     quantum_type=UndefinedQuantum;
     quantum_scale=1.0;
     format=(char) ReadBlobByte(image);
@@ -416,8 +426,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             Assign a value to the specified keyword.
           */
           if (LocaleCompare(keyword,"depth") == 0)
-            packet_size=StringToUnsignedLong(value);
-          (void) packet_size;
+            (void) StringToUnsignedLong(value);
           if (LocaleCompare(keyword,"height") == 0)
             image->rows=StringToUnsignedLong(value);
           if (LocaleCompare(keyword,"maxval") == 0)
@@ -427,23 +436,23 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               if (LocaleCompare(value,"BLACKANDWHITE") == 0)
                 {
-                  (void) SetImageColorspace(image,GRAYColorspace,exception);
+                  colorspace=GRAYColorspace;
                   quantum_type=GrayQuantum;
                 }
               if (LocaleCompare(value,"BLACKANDWHITE_ALPHA") == 0)
                 {
-                  (void) SetImageColorspace(image,GRAYColorspace,exception);
+                  colorspace=GRAYColorspace;
                   image->alpha_trait=BlendPixelTrait;
                   quantum_type=GrayAlphaQuantum;
                 }
               if (LocaleCompare(value,"GRAYSCALE") == 0)
                 {
+                  colorspace=GRAYColorspace;
                   quantum_type=GrayQuantum;
-                  (void) SetImageColorspace(image,GRAYColorspace,exception);
                 }
               if (LocaleCompare(value,"GRAYSCALE_ALPHA") == 0)
                 {
-                  (void) SetImageColorspace(image,GRAYColorspace,exception);
+                  colorspace=GRAYColorspace;
                   image->alpha_trait=BlendPixelTrait;
                   quantum_type=GrayAlphaQuantum;
                 }
@@ -454,12 +463,12 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 }
               if (LocaleCompare(value,"CMYK") == 0)
                 {
-                  (void) SetImageColorspace(image,CMYKColorspace,exception);
+                  colorspace=CMYKColorspace;
                   quantum_type=CMYKQuantum;
                 }
               if (LocaleCompare(value,"CMYK_ALPHA") == 0)
                 {
-                  (void) SetImageColorspace(image,CMYKColorspace,exception);
+                  colorspace=CMYKColorspace;
                   image->alpha_trait=BlendPixelTrait;
                   quantum_type=CMYKAQuantum;
                 }
@@ -484,9 +493,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       {
-        comment_info.comment=DestroyString(comment_info.comment); \
+        if (comment_info.comment != (char *) NULL)
+          comment_info.comment=DestroyString(comment_info.comment);
+        if (quantum_info != (QuantumInfo *) NULL)
+          quantum_info=DestroyQuantumInfo(quantum_info);
         return(DestroyImageList(image));
       }
+    if (colorspace != UndefinedColorspace)
+      (void) SetImageColorspace(image,colorspace,exception);
     (void) ResetImagePixels(image,exception);
     /*
       Convert PNM pixels to runextent-encoded MIFF packets.
@@ -611,6 +625,12 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             pixel=ScaleAnyToQuantum(PNMInteger(image,&comment_info,10,
               exception),max_value);
             SetPixelBlue(image,pixel,q);
+            if ((is_gray != MagickFalse) &&
+                (IsPixelGray(image,q) == MagickFalse))
+              is_gray=MagickFalse;
+            if ((is_mono != MagickFalse) &&
+                (IsPixelMonochrome(image,q) == MagickFalse))
+              is_mono=MagickFalse;
             q+=GetPixelChannels(image);
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
@@ -625,6 +645,10 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (EOFBlob(image) != MagickFalse)
             break;
         }
+        if (is_gray != MagickFalse)
+          image->type=GrayscaleType;
+        if (is_mono != MagickFalse)
+          image->type=BilevelType;
         break;
       }
       case '4':
@@ -747,7 +771,15 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
             default:
             {
-              if (image->depth <= 8)
+              switch (image->depth)
+              {
+                case 1:
+                case 2:
+                case 3:
+                case 4:
+                case 5:
+                case 6:
+                case 7:
                 {
                   unsigned char
                     pixel;
@@ -758,8 +790,15 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     SetPixelGray(image,ScaleAnyToQuantum(pixel,max_value),q);
                     q+=GetPixelChannels(image);
                   }
+                  break;
                 }
-              else if (image->depth <= 16)
+                case 9:
+                case 10:
+                case 11:
+                case 12:
+                case 13:
+                case 14:
+                case 15:
                 {
                   unsigned short
                     pixel;
@@ -770,8 +809,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     SetPixelGray(image,ScaleAnyToQuantum(pixel,max_value),q);
                     q+=GetPixelChannels(image);
                   }
+                  break;
                 }
-              else
+                default:
                 {
                   unsigned int
                     pixel;
@@ -782,7 +822,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     SetPixelGray(image,ScaleAnyToQuantum(pixel,max_value),q);
                     q+=GetPixelChannels(image);
                   }
+                  break;
                 }
+              }
               break;
             }
           }
@@ -800,7 +842,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Convert PNM raster image to pixel packets.
         */
         quantum_type=RGBQuantum;
-        extent=3*(image->depth <= 8 ? 1 : image->depth <= 16 ? 2 : 4)*
+        extent=3*(size_t) (image->depth <= 8 ? 1 : image->depth <= 16 ? 2 : 4)*
           image->columns;
         quantum_info=AcquireQuantumInfo(image_info,image);
         if (quantum_info == (QuantumInfo *) NULL)
@@ -845,6 +887,36 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           p=(unsigned char *) stream;
           switch (image->depth)
           {
+            case 1:
+            case 2:
+            case 3:
+            case 4:
+            case 5:
+            case 6:
+            case 7:
+            {
+              unsigned char
+                pixel;
+
+              for (x=0; x < (ssize_t) image->columns; x++)
+              {
+                p=PushCharPixel(p,&pixel);
+                SetPixelRed(image,ScaleAnyToQuantum(pixel,max_value),q);
+                p=PushCharPixel(p,&pixel);
+                SetPixelGreen(image,ScaleAnyToQuantum(pixel,max_value),q);
+                p=PushCharPixel(p,&pixel);
+                SetPixelBlue(image,ScaleAnyToQuantum(pixel,max_value),q);
+                SetPixelAlpha(image,OpaqueAlpha,q);
+                if ((is_gray != MagickFalse) &&
+                    (IsPixelGray(image,q) == MagickFalse))
+                  is_gray=MagickFalse;
+                if ((is_mono != MagickFalse) &&
+                    (IsPixelMonochrome(image,q) == MagickFalse))
+                  is_mono=MagickFalse;
+                q+=GetPixelChannels(image);
+              }
+              break;
+            }
             case 8:
             {
               for (x=0; x < (ssize_t) image->columns; x++)
@@ -853,6 +925,42 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 SetPixelGreen(image,ScaleCharToQuantum(*p++),q);
                 SetPixelBlue(image,ScaleCharToQuantum(*p++),q);
                 SetPixelAlpha(image,OpaqueAlpha,q);
+                if ((is_gray != MagickFalse) &&
+                    (IsPixelGray(image,q) == MagickFalse))
+                  is_gray=MagickFalse;
+                if ((is_mono != MagickFalse) &&
+                    (IsPixelMonochrome(image,q) == MagickFalse))
+                  is_mono=MagickFalse;
+                q+=GetPixelChannels(image);
+              }
+              break;
+            }
+            case 9:
+            case 10:
+            case 11:
+            case 12:
+            case 13:
+            case 14:
+            case 15:
+            {
+              unsigned short
+                pixel;
+
+              for (x=0; x < (ssize_t) image->columns; x++)
+              {
+                p=PushShortPixel(MSBEndian,p,&pixel);
+                SetPixelRed(image,ScaleAnyToQuantum(pixel,max_value),q);
+                p=PushShortPixel(MSBEndian,p,&pixel);
+                SetPixelGreen(image,ScaleAnyToQuantum(pixel,max_value),q);
+                p=PushShortPixel(MSBEndian,p,&pixel);
+                SetPixelBlue(image,ScaleAnyToQuantum(pixel,max_value),q);
+                SetPixelAlpha(image,OpaqueAlpha,q);
+                if ((is_gray != MagickFalse) &&
+                    (IsPixelGray(image,q) == MagickFalse))
+                  is_gray=MagickFalse;
+                if ((is_mono != MagickFalse) &&
+                    (IsPixelMonochrome(image,q) == MagickFalse))
+                  is_mono=MagickFalse;
                 q+=GetPixelChannels(image);
               }
               break;
@@ -871,6 +979,12 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 p=PushShortPixel(MSBEndian,p,&pixel);
                 SetPixelBlue(image,ScaleShortToQuantum(pixel),q);
                 SetPixelAlpha(image,OpaqueAlpha,q);
+                if ((is_gray != MagickFalse) &&
+                    (IsPixelGray(image,q) == MagickFalse))
+                  is_gray=MagickFalse;
+                if ((is_mono != MagickFalse) &&
+                    (IsPixelMonochrome(image,q) == MagickFalse))
+                  is_mono=MagickFalse;
                 q+=GetPixelChannels(image);
               }
               break;
@@ -889,63 +1003,38 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 p=PushLongPixel(MSBEndian,p,&pixel);
                 SetPixelBlue(image,ScaleLongToQuantum(pixel),q);
                 SetPixelAlpha(image,OpaqueAlpha,q);
+                if ((is_gray != MagickFalse) &&
+                    (IsPixelGray(image,q) == MagickFalse))
+                  is_gray=MagickFalse;
+                if ((is_mono != MagickFalse) &&
+                    (IsPixelMonochrome(image,q) == MagickFalse))
+                  is_mono=MagickFalse;
                 q+=GetPixelChannels(image);
               }
               break;
             }
             default:
             {
-              if (image->depth <= 8)
-                {
-                  unsigned char
-                    pixel;
+              unsigned int
+                pixel;
 
-                  for (x=0; x < (ssize_t) image->columns; x++)
-                  {
-                    p=PushCharPixel(p,&pixel);
-                    SetPixelRed(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    p=PushCharPixel(p,&pixel);
-                    SetPixelGreen(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    p=PushCharPixel(p,&pixel);
-                    SetPixelBlue(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    SetPixelAlpha(image,OpaqueAlpha,q);
-                    q+=GetPixelChannels(image);
-                  }
-                }
-              else if (image->depth <= 16)
-                {
-                  unsigned short
-                    pixel;
-
-                  for (x=0; x < (ssize_t) image->columns; x++)
-                  {
-                    p=PushShortPixel(MSBEndian,p,&pixel);
-                    SetPixelRed(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    p=PushShortPixel(MSBEndian,p,&pixel);
-                    SetPixelGreen(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    p=PushShortPixel(MSBEndian,p,&pixel);
-                    SetPixelBlue(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    SetPixelAlpha(image,OpaqueAlpha,q);
-                    q+=GetPixelChannels(image);
-                  }
-                }
-              else
-                {
-                  unsigned int
-                    pixel;
-
-                  for (x=0; x < (ssize_t) image->columns; x++)
-                  {
-                    p=PushLongPixel(MSBEndian,p,&pixel);
-                    SetPixelRed(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    p=PushLongPixel(MSBEndian,p,&pixel);
-                    SetPixelGreen(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    p=PushLongPixel(MSBEndian,p,&pixel);
-                    SetPixelBlue(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    SetPixelAlpha(image,OpaqueAlpha,q);
-                    q+=GetPixelChannels(image);
-                  }
-                }
+              for (x=0; x < (ssize_t) image->columns; x++)
+              {
+                p=PushLongPixel(MSBEndian,p,&pixel);
+                SetPixelRed(image,ScaleAnyToQuantum(pixel,max_value),q);
+                p=PushLongPixel(MSBEndian,p,&pixel);
+                SetPixelGreen(image,ScaleAnyToQuantum(pixel,max_value),q);
+                p=PushLongPixel(MSBEndian,p,&pixel);
+                SetPixelBlue(image,ScaleAnyToQuantum(pixel,max_value),q);
+                SetPixelAlpha(image,OpaqueAlpha,q);
+                if ((is_gray != MagickFalse) &&
+                    (IsPixelGray(image,q) == MagickFalse))
+                  is_gray=MagickFalse;
+                if ((is_mono != MagickFalse) &&
+                    (IsPixelMonochrome(image,q) == MagickFalse))
+                  is_mono=MagickFalse;
+                q+=GetPixelChannels(image);
+              }
               break;
             }
           }
@@ -953,6 +1042,10 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (sync == MagickFalse)
             break;
         }
+        if (is_gray != MagickFalse)
+          image->type=GrayscaleType;
+        if (is_mono != MagickFalse)
+          image->type=BilevelType;
         quantum_info=DestroyQuantumInfo(quantum_info);
         break;
       }
@@ -1045,11 +1138,20 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 case GrayQuantum:
                 case GrayAlphaQuantum:
                 {
-                  if (image->depth <= 8)
+                  switch (image->depth)
+                  {
+                    case 1:
+                    case 2:
+                    case 3:
+                    case 4:
+                    case 5:
+                    case 6:
+                    case 7:
+                    case 8:
                     {
                       unsigned char
                         pixel;
-
+ 
                       for (x=0; x < (ssize_t) image->columns; x++)
                       {
                         p=PushCharPixel(p,&pixel);
@@ -1068,8 +1170,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
-                  else if (image->depth <= 16)
+                    case 9:
+                    case 10:
+                    case 11:
+                    case 12:
+                    case 13:
+                    case 14:
+                    case 15:
+                    case 16:
                     {
                       unsigned short
                         pixel;
@@ -1088,8 +1198,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
-                  else
+                    default:
                     {
                       unsigned int
                         pixel;
@@ -1109,12 +1220,22 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         q+=GetPixelChannels(image);
                       }
                     }
+                  }
                   break;
                 }
                 case CMYKQuantum:
                 case CMYKAQuantum:
                 {
-                  if (image->depth <= 8)
+                  switch (image->depth)
+                  {
+                    case 1:
+                    case 2:
+                    case 3:
+                    case 4:
+                    case 5:
+                    case 6:
+                    case 7:
+                    case 8:
                     {
                       unsigned char
                         pixel;
@@ -1141,8 +1262,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
-                  else if (image->depth <= 16)
+                    case 9:
+                    case 10:
+                    case 11:
+                    case 12:
+                    case 13:
+                    case 14:
+                    case 15:
+                    case 16:
                     {
                       unsigned short
                         pixel;
@@ -1169,8 +1298,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
-                  else
+                    default:
                     {
                       unsigned int
                         pixel;
@@ -1197,13 +1327,24 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                               max_value),q);
                           }
                         q+=GetPixelChannels(image);
+                      }
+                      break;
                     }
                   }
                   break;
                 }
                 default:
                 {
-                  if (image->depth <= 8)
+                  switch (image->depth)
+                  {
+                    case 1:
+                    case 2:
+                    case 3:
+                    case 4:
+                    case 5:
+                    case 6:
+                    case 7:
+                    case 8:
                     {
                       unsigned char
                         pixel;
@@ -1227,8 +1368,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
-                  else if (image->depth <= 16)
+                    case 9:
+                    case 10:
+                    case 11:
+                    case 12:
+                    case 13:
+                    case 14:
+                    case 15:
+                    case 16:
                     {
                       unsigned short
                         pixel;
@@ -1252,8 +1401,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
-                  else
+                    default:
                     {
                       unsigned int
                         pixel;
@@ -1278,7 +1428,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           }
                         q+=GetPixelChannels(image);
                       }
+                      break;
                     }
+                  }
                   break;
                 }
               }
@@ -1484,6 +1636,10 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   } while ((count == 1) && (format == 'P'));
   (void) CloseBlob(image);
+  if (comment_info.comment != (char *) NULL)
+    comment_info.comment=DestroyString(comment_info.comment);
+  if (quantum_info != (QuantumInfo *) NULL)
+    quantum_info=DestroyQuantumInfo(quantum_info);
   if (status == MagickFalse)
     return(DestroyImageList(image));
   return(GetFirstImageInList(image));
@@ -1648,18 +1804,18 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
   QuantumType
     quantum_type;
 
-  unsigned char
-    *q;
-
   size_t
     extent,
-    imageListLength,
+    number_scenes,
     packet_size;
 
   ssize_t
     count,
     y;
 
+  unsigned char
+    *q;
+
   /*
     Open output image file.
   */
@@ -1667,15 +1823,15 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   scene=0;
-  imageListLength=GetImageListLength(image);
+  number_scenes=GetImageListLength(image);
   do
   {
     QuantumAny
@@ -1710,7 +1866,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
         format='F';
         if (image_info->type == TrueColorType)
           break;
-        if (IdentifyImageType(image,exception) == GrayscaleType)
+        if (IdentifyImageCoderGray(image,exception) != MagickFalse)
           format='f';
         break;
       }
@@ -1728,7 +1884,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
         format='H';
         if (image_info->type == TrueColorType)
           break;
-        if (IdentifyImageType(image,exception) == GrayscaleType)
+        if (IdentifyImageCoderGray(image,exception) != MagickFalse)
           format='h';
         break;
       }
@@ -1741,8 +1897,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
         format='6';
         if (image_info->type == TrueColorType)
           break;
-        type=IdentifyImageType(image,exception);
-        if (type == GrayscaleType)
+        type=IdentifyImageCoderGrayType(image,exception);
+        if (IsGrayImageType(type) != MagickFalse)
           {
             format='5';
             if (image_info->compression == NoCompression)
@@ -1753,8 +1909,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                 if (image_info->compression == NoCompression)
                   format='1';
               }
-            break;
           }
+        break;
       }
       default:
       {
@@ -1817,7 +1973,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           {
             packet_size=1;
             (void) CopyMagickString(type,"GRAYSCALE",MagickPathExtent);
-            if (IdentifyImageMonochrome(image,exception) != MagickFalse)
+            if (GetQuantumRange(image->depth) == 1)
               (void) CopyMagickString(type,"BLACKANDWHITE",MagickPathExtent);
             break;
           }
@@ -1980,7 +2136,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
         /*
           Convert image to a PNM image.
         */
-        (void) TransformImageColorspace(image,sRGBColorspace,exception);
+        if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+          (void) TransformImageColorspace(image,sRGBColorspace,exception);
         if (image->depth <= 8)
           (void) WriteBlobString(image,"255\n");
         else
@@ -2213,7 +2370,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
         /*
           Convert image to a PNM image.
         */
-        (void) TransformImageColorspace(image,sRGBColorspace,exception);
+        if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+          (void) TransformImageColorspace(image,sRGBColorspace,exception);
         if (image->depth > 32)
           image->depth=32;
         (void) FormatLocaleString(buffer,MagickPathExtent,"%.20g\n",(double)
@@ -2640,7 +2798,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
+    status=SetImageProgress(image,SaveImagesTag,scene++,number_scenes);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
