@@ -17,7 +17,7 @@
 %                                January 2014                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 2014 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -61,7 +61,6 @@
 #include "MagickCore/option.h"
 #include "MagickCore/pixel.h"
 #include "MagickCore/pixel-accessor.h"
-#include "MagickCore/pixel-private.h"
 #include "MagickCore/prepress.h"
 #include "MagickCore/property.h"
 #include "MagickCore/quantum-private.h"
@@ -74,6 +73,7 @@
 #include "MagickCore/utility.h"
 #include "MagickCore/version.h"
 #include "MagickCore/module.h"
+#include "coders/coders-private.h"
 
 /*
   Typedef declarations.
@@ -329,7 +329,7 @@ static ChannelStatistics *GetLocationStatistics(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   channel_statistics=(ChannelStatistics *) AcquireQuantumMemory(
     MaxPixelChannels+1,sizeof(*channel_statistics));
@@ -367,9 +367,6 @@ static ChannelStatistics *GetLocationStatistics(const Image *image,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      ssize_t
-        i;
-
       if (GetPixelReadMask(image,p) <= (QuantumRange/2))
         {
           p+=GetPixelChannels(image);
@@ -853,7 +850,7 @@ static void EncodeIptcProfile(FILE *file,const StringInfo *profile)
         value->values_length=0;
         values[count++]=value;
       }
-    length=(size_t) (GetStringInfoDatum(profile)[i++] << 8);
+    length=((size_t) GetStringInfoDatum(profile)[i++] << 8);
     length|=GetStringInfoDatum(profile)[i++];
     attribute=(char *) NULL;
     if (~length >= (MagickPathExtent-1))
@@ -952,31 +949,30 @@ static MagickBooleanType EncodeImageAttributes(Image *image,FILE *file,
   MagickBooleanType
     ping;
 
-  const Quantum
-    *p;
-
-  ssize_t
-    i,
-    x;
-
   size_t
     depth,
     distance,
     scale;
 
   ssize_t
+    i,
+    x,
     y;
 
+  struct stat
+    properties;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   *format='\0';
   elapsed_time=GetElapsedTime(&image->timer);
   user_time=GetUserTime(&image->timer);
   GetTimerInfo(&image->timer);
-  p=GetVirtualPixels(image,0,0,1,1,exception);
-  ping=p == (const Quantum *) NULL ? MagickTrue : MagickFalse;
+  ping=MagickTrue;
+  if (GetVirtualPixels(image,0,0,1,1,exception) != (const Quantum *) NULL)
+    ping=MagickFalse;
   (void) ping;
   (void) SignatureImage(image,exception);
   (void) FormatLocaleFile(file,"---\n");
@@ -1002,12 +998,17 @@ static MagickBooleanType EncodeImageAttributes(Image *image,FILE *file,
           YAMLFormatLocaleFile(file,"    baseName: %s\n",filename);
         }
     }
+  properties=(*GetBlobProperties(image));
+  if (properties.st_mode != 0)
+    (void) FormatLocaleFile(file,"    permissions: %d%d%d\n",
+      (properties.st_mode >> 6) & 0x07,(properties.st_mode >> 3) & 0x07,
+      (properties.st_mode >> 0) & 0x07);
   YAMLFormatLocaleFile(file,"    format: %s\n",image->magick);
   magick_info=GetMagickInfo(image->magick,exception);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetMagickDescription(magick_info) != (const char *) NULL))
     YAMLFormatLocaleFile(file,"    formatDescription: %s\n",
-      image->magick);
+      GetMagickDescription(magick_info));
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetMagickMimeType(magick_info) != (const char *) NULL))
     YAMLFormatLocaleFile(file,"    mimeType: %s\n",GetMagickMimeType(
@@ -1037,7 +1038,7 @@ static MagickBooleanType EncodeImageAttributes(Image *image,FILE *file,
     }
   YAMLFormatLocaleFile(file,"    units: %s\n",CommandOptionToMnemonic(
     MagickResolutionOptions,(ssize_t) image->units));
-  type=IdentifyImageType(image,exception);
+  type=IdentifyImageCoderType(image,exception);
   YAMLFormatLocaleFile(file,"    type: %s\n",CommandOptionToMnemonic(
     MagickTypeOptions,(ssize_t) type));
   if (image->type != type)
@@ -1061,12 +1062,12 @@ static MagickBooleanType EncodeImageAttributes(Image *image,FILE *file,
         max_locations;
 
       StatisticType
-        type;
+        statistic_type;
 
       /*
         Display minimum, maximum, or mean pixel locations.
       */
-      type=(StatisticType) ParseCommandOption(MagickStatisticOptions,
+      statistic_type=(StatisticType) ParseCommandOption(MagickStatisticOptions,
         MagickFalse,locate);
       limit=GetImageArtifact(image,"identify:limit");
       if (limit == (const char *) NULL)
@@ -1074,43 +1075,44 @@ static MagickBooleanType EncodeImageAttributes(Image *image,FILE *file,
       max_locations=0;
       if (limit != (const char *) NULL)
         max_locations=StringToUnsignedLong(limit);
-      channel_statistics=GetLocationStatistics(image,type,exception);
+      channel_statistics=GetLocationStatistics(image,statistic_type,exception);
       if (channel_statistics == (ChannelStatistics *) NULL)
         return(MagickFalse);
       (void) FormatLocaleFile(file,"    channel%s: \n",locate);
       if (image->alpha_trait != UndefinedPixelTrait)
         (void) PrintChannelLocations(file,image,AlphaPixelChannel,"alpha",
-          type,max_locations,MagickTrue,channel_statistics);
+          statistic_type,max_locations,MagickTrue,channel_statistics);
       switch (image->colorspace)
       {
         case RGBColorspace:
         default:
         {
           (void) PrintChannelLocations(file,image,RedPixelChannel,"red",
-            type,max_locations,MagickTrue,channel_statistics);
+            statistic_type,max_locations,MagickTrue,channel_statistics);
           (void) PrintChannelLocations(file,image,GreenPixelChannel,"green",
-            type,max_locations,MagickTrue,channel_statistics);
+            statistic_type,max_locations,MagickTrue,channel_statistics);
           (void) PrintChannelLocations(file,image,BluePixelChannel,"blue",
-            type,max_locations,MagickFalse,channel_statistics);
+            statistic_type,max_locations,MagickFalse,channel_statistics);
           break;
         }
         case CMYKColorspace:
         {
           (void) PrintChannelLocations(file,image,CyanPixelChannel,"cyan",
-            type,max_locations,MagickTrue,channel_statistics);
+            statistic_type,max_locations,MagickTrue,channel_statistics);
           (void) PrintChannelLocations(file,image,MagentaPixelChannel,
-            "magenta",type,max_locations,MagickTrue,channel_statistics);
+            "magenta",statistic_type,max_locations,MagickTrue,
+            channel_statistics);
           (void) PrintChannelLocations(file,image,YellowPixelChannel,"yellow",
-            type,max_locations,MagickTrue,channel_statistics);
+            statistic_type,max_locations,MagickTrue,channel_statistics);
           (void) PrintChannelLocations(file,image,BlackPixelChannel,"black",
-            type,max_locations,MagickFalse,channel_statistics);
+            statistic_type,max_locations,MagickFalse,channel_statistics);
           break;
         }
         case LinearGRAYColorspace:
         case GRAYColorspace:
         {
           (void) PrintChannelLocations(file,image,GrayPixelChannel,"gray",
-            type,max_locations,MagickFalse,channel_statistics);
+            statistic_type,max_locations,MagickFalse,channel_statistics);
           break;
         }
       }
@@ -1704,7 +1706,7 @@ static MagickBooleanType WriteYAMLImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBlobMode,exception);
   if (status == MagickFalse)
